# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

from elftools.elf.dynamic import DynamicSection
from elftools.elf.elffile import ELFFile

from Namcap.ruleclass import TarballRule
from Namcap.util import is_elf

allowed = ["/usr/lib", "/usr/lib32", "/lib", "$ORIGIN", "${ORIGIN}"]
allowed_toplevels = [s + "/" for s in allowed]
warn = ["/usr/local/lib"]


def get_rpaths(fileobj):
    elffile = ELFFile(fileobj)
    for section in elffile.iter_sections():
        if not isinstance(section, DynamicSection):
            continue
        for tag in section.iter_tags("DT_RPATH"):
            rpaths = tag.rpath
            rpaths = rpaths.split(":")
            for path in rpaths:
                yield path


class package(TarballRule):
    name = "rpath"
    description = "Verifies correct and secure RPATH for files."

    def analyze(self, pkginfo, tar):
        for entry in tar:
            if not entry.isfile():
                continue

            # is it an ELF file ?
            fileobj = tar.extractfile(entry)
            if not is_elf(fileobj):
                continue

            for path in get_rpaths(fileobj):
                path_ok = path in allowed
                for allowed_toplevel in allowed_toplevels:
                    if path.startswith(allowed_toplevel):
                        path_ok = True

                if not path_ok:
                    if path in warn:
                        self.warnings.append(("insecure-rpath %s %s", (path, entry.name)))
                    else:
                        self.errors.append(("insecure-rpath %s %s", (path, entry.name)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

from Namcap.ruleclass import PkgbuildRule


class RedundantMakedepends(PkgbuildRule):
    """
    This rule checks for make dependencies that are already
    included as dependencies.
    """

    name = "redundant_makedepends"
    description = "Check for redundant make dependencies"

    def analyze(self, pkginfo, pkgbuild):
        redundant_makedeps: list[str] = []

        if "makedepends" not in pkginfo:
            return
        if "depends" not in pkginfo:
            return
        redundant_makedeps.extend(set(pkginfo["makedepends"]) & set(pkginfo["depends"]))

        for i in redundant_makedeps:
            self.warnings.append(("redundant-makedep %s", (i,)))


class VCSMakedepends(PkgbuildRule):
    """
    This rule checks for missing VCS make dependencies.
    """

    name = "vcs_makedepends"
    description = "Verify make dependencies for VCS sources"

    def analyze(self, pkginfo, pkgbuild):
        vcs = {
            "bzr": "breezy",
            "git": "git",
            "hg": "mercurial",
            "svn": "subversion",
        }
        missing = []
        protocols = set()

        if "source" not in pkginfo:
            return

        for s in pkginfo["source"]:
            p = s.split("::", 1)[-1]
            p = p.split("://", 1)[0]
            p = p.split("+", 1)[0]
            if p in vcs:
                protocols.add(p)

        if not protocols:
            return

        for v in protocols:
            d = vcs[v]
            if "makedepends" not in pkginfo:
                missing.append(d)
                continue
            if d not in pkginfo["makedepends"]:
                missing.append(d)

        for i in missing:
            self.warnings.append(("missing-vcs-makedeps %s", (i,)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

from Namcap.ruleclass import TarballRule


class package(TarballRule):
    name = "fileownership"
    description = "Checks file ownership."

    def analyze(self, pkginfo, tar):
        for i in tar.getmembers():
            if i.uname != "root" or i.gname != "root":
                uname = ""
                gname = ""
                if i.uname == "":
                    uname = str(i.uid)
                else:
                    uname = i.uname
                if i.gname == "":
                    gname = str(i.gid)
                else:
                    gname = i.gname
                self.errors.append(("incorrect-owner %s (%s:%s)", (i.name, uname, gname)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import re

from Namcap.ruleclass import TarballRule


class package(TarballRule):
    name = "libtool"
    description = "Checks for libtool (*.la) files."

    def analyze(self, pkginfo, tar):
        for i in tar.getnames():
            if re.search(r"\.la$", i) is not None:
                self.warnings.append(("libtool-file-present %s", i))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"""Checks dependencies resulting from pkg-config files."""

import os
import shutil
import subprocess
import tempfile
from collections import defaultdict

import Namcap.package
from Namcap.ruleclass import TarballRule


def scanpcfiles(tar, pclist):
    """
    Find dependencies of pkg-config files
    """

    pkg_pc_files = [f for f in tar.getnames() if ".pc" in f]
    for f in pkg_pc_files:
        if f.startswith(("usr/lib/pkgconfig", "usr/share/pkgconfig")):
            pcname = f.replace("usr/lib/pkgconfig/", "").replace("usr/share/pkgconfig/", "").replace(".pc", "")
            pkgconfig_command = "pkg-config"
        elif f.startswith("usr/lib32/pkgconfig"):
            pcname = f.replace("usr/lib32/pkgconfig/", "").replace(".pc", "")
            pkgconfig_command = "i686-pc-linux-gnu-pkg-config"
        else:
            continue

        tmpdir = tempfile.mkdtemp()
        tar.extract(f, tmpdir)
        var = subprocess.Popen(
            [
                pkgconfig_command,
                "--maximum-traverse-depth",
                "1",
                "--print-requires",
                "--print-requires-private",
                pcname,
            ],
            env={"LANG": "C", "PKG_CONFIG_LIBDIR": tmpdir + "/" + os.path.dirname(f)},
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        ).communicate()
        shutil.rmtree(tmpdir)
        for j in var[0].decode("ascii").splitlines():
            # Remove version numbers
            pc_pkg = j.split(" ", 1)[0]
            if pc_pkg is not None:
                var = subprocess.Popen(
                    [pkgconfig_command, "--maximum-traverse-depth", "1", "--path", pc_pkg],
                    env={"LANG": "C"},
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ).communicate()
                pc_path = var[0].decode("ascii").splitlines()
                if pc_path:
                    pclist[pc_path[0][1:]].add(f)
                else:
                    pclist[pc_pkg + ".pc"].add(f)


def finddepends(pclist):
    """
    Find packages owning a list of pkg-config files

    Returns:
      dependlist -- a dictionary { package => set(pclist) }
      orphans -- the list of pkg-config files without owners
    """
    dependlist = defaultdict(set)
    knownpcs = set(pclist)
    foundpcs = set()

    for pkg in Namcap.package.get_installed_packages():
        for fname, fsize, fmode in pkg.files:
            for k in knownpcs:
                if fname == k:
                    dependlist[pkg.name].add(k)
                    foundpcs.add(k)

    orphans = list(knownpcs - foundpcs)
    return dependlist, orphans


class PkgConfigDependenciesRule(TarballRule):
    name = "pcdepends"
    description = "Checks dependencies caused by pkg-config files"

    def analyze(self, pkginfo, tar):
        pclist: dict[str, set[str]] = defaultdict(set)
        dependlist = {}

        # Detect dependencies from pkg-config files
        scanpcfiles(tar, pclist)

        # Find the packages which contain the pkg-config files
        dependlist, orphans = finddepends(pclist)

        # Handle "no package associated" errors
        self.warnings.extend([("pkgconf-no-package-associated %s %s", (i, str(list(pclist[i])))) for i in orphans])

        # Print deps
        for pkg, libraries in dependlist.items():
            if isinstance(libraries, set):
                files = list(libraries)
                needing = set().union(*[pclist[lib] for lib in libraries])
                reasons = pkginfo.detected_deps.setdefault(pkg, [])
                reasons.append(("pkgconf-needed %s %s", (str(files), str(list(needing)))))
                self.infos.append(("pkgconf-dependence %s in %s", (pkg, str(files))))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import Namcap.rules.badbackups as module
from Namcap.tests.pkgbuild_test import PkgbuildTest


class NamcapBadBackupsTest(PkgbuildTest):
    pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
backup=('/etc/rc.conf')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""

    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = module.package

    def test_example1(self):
        # Example 1
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [("backups-preceding-slashes", ())])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import Namcap.rules.makedepends as module
from Namcap.tests.pkgbuild_test import PkgbuildTest


class NamcapRedundantMakedependsTest(PkgbuildTest):
    pkgbuild1 = """
# Maintainer: Arch Linux <archlinux at example.com>
# Contributor: Arch Linux <archlinux at example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
url="http://www.example.com/"
arch=('i686' 'x86_64')
depends=('lib1' 'lib2' 'lib3')
makedepends=('lib1' 'lib2' 'lib4')
license=('GPL-3.0-or-later')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  true
}

package() {
  true
}
"""
    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = module.RedundantMakedepends

    def test_example1(self):
        # Example 1
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [])
        self.assertEqual(set(r.warnings), set(("redundant-makedep %s", (i,)) for i in ["lib1", "lib2"]))
        self.assertEqual(r.infos, [])


class NamcapVCSMakedependsTest(PkgbuildTest):
    pkgbuild1 = """
# Maintainer: Arch Linux <archlinux at example.com>
# Contributor: Arch Linux <archlinux at example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
url="http://www.example.com/"
arch=('i686' 'x86_64')
depends=()
makedepends=()
license=('GPL-3.0-or-later')
options=('!libtool')
source=(name::bzr+https://example.com/pub/mypackage
        name::git://example.com/pub/mypackage
        hg+https://example.com/pub/mypackage
        svn://example.com/pub/mypackage)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  true
}

package() {
  true
}
"""
    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = module.VCSMakedepends

    def test_example1(self):
        # Example 1
        makedeps = ["breezy", "git", "mercurial", "subversion"]
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [])
        self.assertEqual(set(r.warnings), set(("missing-vcs-makedeps %s", (i,)) for i in makedeps))
        self.assertEqual(r.infos, [])

    def test_example2(self):
        # Example 2
        r = self.run_on_pkg(self.pkgbuild1 + "source=(gitsomething)")
        self.assertEqual(r.errors, [])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

from Namcap.ruleclass import TarballRule


class systemdlocationRule(TarballRule):
    name = "systemdlocation"
    description = "Checks for systemd files in /etc/systemd/system/"

    def analyze(self, pkginfo, tar):
        # don't have this warning for the systemd package
        if "name" in pkginfo:
            if pkginfo["name"] == "systemd":
                return
        # don't have this warning for packages that provides systemd
        if "provides" in pkginfo:
            if "systemd" in pkginfo["provides"]:
                return
        for entry in tar:
            # ignore the actual directory, as that's handled by emptydirs
            if entry.isdir():
                continue

            name = os.path.normpath(entry.name)

            # check for files in /etc/systemd/system/
            if name.startswith("etc/systemd/system/"):
                self.warnings.append(("systemd-location %s", name))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.externalhooks
from Namcap.tests.makepkg import MakepkgTest


class ExternalHooksTest(MakepkgTest):
    pkgbuild_generic = """
pkgname=__namcap_test_externalhooks
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('any')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
options=(emptydirs)
install=sample.install
source=()
build() {
  true
}
package() {
  true
}

"""
    install_bad = """
post_install() {
  update-desktop-database -q
  gtk-update-icon-cache -q -t -f usr/share/icons/hicolor
}
"""
    install_good = """
post_install() {
  true
}
"""

    def test_bad_install(self):
        pkgfile = "__namcap_test_externalhooks-1.0-1-any.pkg.tar"
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild_generic)
        with open(os.path.join(self.tmpdir, "sample.install"), "w") as f:
            f.write(self.install_bad)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(
            os.path.join(self.tmpdir, pkgfile), Namcap.rules.externalhooks.ExternalHooksRule
        )
        self.assertEqual(r.errors, [])
        self.assertEqual(
            r.warnings,
            [("external-hooks %s", ("update-desktop-database",)), ("external-hooks %s", ("gtk-update-icon-cache",))],
        )
        self.assertEqual(r.infos, [])

    def test_good_install(self):
        pkgfile = "__namcap_test_externalhooks-1.0-1-any.pkg.tar"
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild_generic)
        with open(os.path.join(self.tmpdir, "sample.install"), "w") as f:
            f.write(self.install_good)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(
            os.path.join(self.tmpdir, pkgfile), Namcap.rules.externalhooks.ExternalHooksRule
        )
        self.assertEqual(r.errors, [])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"""
This module defines the base classes from which Namcap rules are derived
and how they are meant to be used.
"""

from abc import ABC, abstractmethod
from tarfile import TarFile
from typing import TYPE_CHECKING

from .package import PacmanPackage

if TYPE_CHECKING:
    from .types import Diagnostic


class AbstractRule(ABC):
    "The parent class of all rules"

    enable: bool = True

    def __init__(self) -> None:
        self.errors: list[Diagnostic] = []
        self.warnings: list[Diagnostic] = []
        self.infos: list[Diagnostic] = []


class PkgInfoRule(AbstractRule):
    "The parent class of rules that process package metadata"

    @abstractmethod
    def analyze(self, pkginfo: PacmanPackage, pkgbuild: str | None) -> None: ...


class PkgbuildRule(AbstractRule):
    "The parent class of rules that process PKGBUILDs"

    @abstractmethod
    def analyze(self, pkginfo: PacmanPackage, pkgbuild: str) -> None: ...


class TarballRule(AbstractRule):
    "The parent class of rules that process tarballs"

    @abstractmethod
    def analyze(self, pkginfo: PacmanPackage, tar: TarFile) -> None: ...



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import re

from Namcap.ruleclass import PkgbuildRule


class package(PkgbuildRule):
    name = "invalidstartdir"
    description = "Looks for references to $startdir"

    def analyze(self, pkginfo, tar):
        for i in pkginfo.pkgbuild:
            startdirs = re.split(r'\${?startdir}?"?', i)
            for j in startdirs[1:]:
                if j[:4] != "/pkg" and j[:4] != "/src":
                    self.errors.append(("file-referred-in-startdir", ()))
                elif j[:4] == "/pkg":
                    self.errors.append(("use-pkgdir", ()))
                elif j[:4] == "/src":
                    self.errors.append(("use-srcdir", ()))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.filenames
from Namcap.tests.makepkg import MakepkgTest


class FilenamesTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_nonascii
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p "${pkgdir}/usr/bin/Arch Linux"
  mkdir -p ${pkgdir}/usr/bin/Arch·Linux
  mkdir -p ${pkgdir}/usr/bin/$'Arch\\xb7Linux'
}
"""

    def test_nonascii(self):
        pkgfile = "__namcap_test_nonascii-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w", encoding="utf-8") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.filenames.package)
        self.assertEqual(r.errors, [])
        tag, value1 = r.warnings[0]
        tag, value2 = r.warnings[1]
        self.assertEqual(tag, "invalid-filename")
        self.assertEqual(
            set((value1.encode(errors="surrogateescape"), value2.encode(errors="surrogateescape"))),
            set((b"usr/bin/Arch\xc2\xb7Linux", b"usr/bin/Arch\xb7Linux")),
        )
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os
import shutil
import tempfile
import unittest

import Namcap.package

pkgbuild = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc' 'foobar')
optdepends=('libabc: provides the abc feature')
provides=('yourpackage=0.9')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""


class PkgbuildLoaderTests(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        tmpname = os.path.join(self.tmpdir, "PKGBUILD")
        with open(tmpname, "w") as f:
            f.write(pkgbuild)
        self.pkginfo = Namcap.package.load_from_pkgbuild(tmpname)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_pkgname(self):
        self.assertEqual(self.pkginfo["name"], "mypackage")
        self.assertEqual(self.pkginfo["pkgname"], "mypackage")

    def test_version(self):
        self.assertEqual(self.pkginfo["version"], "1.0-1")

    def test_pkgdesc(self):
        self.assertEqual(self.pkginfo["desc"], "A package")

    def test_url(self):
        self.assertEqual(self.pkginfo["url"], "http://www.example.com/")

    def test_depends(self):
        self.assertEqual(self.pkginfo["depends"], ["glibc", "foobar"])
        self.assertEqual(self.pkginfo["orig_depends"], ["glibc", "foobar"])

    def test_optdeps(self):
        self.assertEqual(self.pkginfo["optdepends"], ["libabc"])
        self.assertEqual(self.pkginfo["orig_optdepends"], ["libabc: provides the abc feature"])

    def test_provides(self):
        self.assertEqual(self.pkginfo["provides"], ["yourpackage"])
        self.assertEqual(self.pkginfo["orig_provides"], ["yourpackage=0.9"])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"""Checks for missing variables in PKGBUILD"""

import re

from Namcap.ruleclass import PkgbuildRule

RE_IS_HEXNUMBER = re.compile(r"[0-9a-f]+")


class ChecksumsRule(PkgbuildRule):
    name = "checksums"
    description = "Verifies checksums are included in a PKGBUILD"

    def analyze(self, pkginfo, tar):
        checksums = [
            ("md5", 32),
            ("sha1", 40),
            ("sha224", 56),
            ("sha256", 64),
            ("sha384", 96),
            ("sha512", 128),
            ("b2", 128),
        ]

        if "source" in pkginfo:
            haschecksums = False
            for sumname, sumlen in checksums:
                if (sumname + "sums") in pkginfo:
                    haschecksums = True
            if not haschecksums:
                self.errors.append(("missing-checksums", ()))
        else:
            pkginfo["source"] = []

        for sumname, sumlen in checksums:
            sumname += "sums"
            if sumname in pkginfo:
                if len(pkginfo["source"]) > len(pkginfo[sumname]):
                    self.errors.append(("not-enough-checksums %s %i needed", (sumname, len(pkginfo["source"]))))
                elif len(pkginfo["source"]) < len(pkginfo[sumname]):
                    self.errors.append(("too-many-checksums %s %i needed", (sumname, len(pkginfo["source"]))))
                for csum in pkginfo[sumname]:
                    if csum == "SKIP":
                        continue
                    if len(csum) != sumlen or not RE_IS_HEXNUMBER.match(csum):
                        self.errors.append(("improper-checksum %s %s", (sumname, csum)))


class TagsRule(PkgbuildRule):
    name = "tags"
    description = "Looks for Maintainer and Contributor comments"

    def analyze(self, pkginfo, tar):
        contributortag = 0
        maintainertag = 0
        for i in pkginfo.pkgbuild:
            if re.match(r"#\s*Contributor\s*:", i):
                contributortag = 1
            if re.match(r"#\s*Maintainer\s*:", i):
                maintainertag = 1

        if contributortag != 1:
            self.infos.append(("missing-contributor", ()))

        if maintainertag != 1:
            self.warnings.append(("missing-maintainer", ()))


class DescriptionRule(PkgbuildRule):
    name = "description"
    description = "Verifies that the description is set in a PKGBUILD"

    def analyze(self, pkginfo, tar):
        if pkginfo.is_split:
            for split_pkginfo in pkginfo.subpackages:
                if "desc" not in split_pkginfo or len(split_pkginfo["desc"]) == 0:
                    self.errors.append(("missing-description %s", (split_pkginfo["name"],)))
        else:
            if "desc" not in pkginfo or len(pkginfo["desc"]) == 0:
                self.errors.append(("missing-description", ()))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"""Checks for bad backup entries"""

import re

from Namcap.ruleclass import PkgbuildRule


class package(PkgbuildRule):
    name = "badbackups"
    description = "Checks for bad backup entries"

    def analyze(self, pkginfo, tar):
        if "backup" in pkginfo:
            for item in pkginfo["backup"]:
                if re.match(r"^/", item) is not None:
                    self.errors.append(("backups-preceding-slashes", ()))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.unusedsodepends
from Namcap.tests.makepkg import MakepkgTest


class UnusedSodependsTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_unusedsodepends
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
options=(!purge !zipman)
build() {
  cd "${srcdir}"
  echo "int main() { return 0; }" > main.c
  /usr/bin/gcc -o main main.c -lm
}
package() {
  install -D -m 755 "$srcdir/main" "$pkgdir/usr/bin/evilprogram"
}
"""

    def test_unusedsodepends_files(self):
        "Package with binaries linking to unused shared libraries"
        pkgfile = "__namcap_test_unusedsodepends-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.unusedsodepends.package)
        self.assertEqual(r.errors, [])
        self.assertEqual(r.warnings, [("unused-sodepend %s %s", ("/usr/lib/libm.so.6", "usr/bin/evilprogram"))])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import re

from Namcap.ruleclass import PkgbuildRule


class package(PkgbuildRule):
    name = "sfurl"
    description = "Checks for proper sourceforge URLs"

    def analyze(self, pkginfo, tar):
        if "source" in pkginfo:
            for source in pkginfo["source"]:
                if re.match(r"(http://|ftp://)\w+.dl.(sourceforge|sf).net", source):
                    self.warnings.append(("specific-sourceforge-mirror", ()))
                if re.match(r"(http://|ftp://)dl.(sourceforge|sf).net", source):
                    self.warnings.append(("using-dl-sourceforge", ()))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.fileownership
from Namcap.tests.makepkg import MakepkgTest


class FileOwnershipTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_fileownership
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/usr/bin
  touch ${pkgdir}/usr/bin/program
  chown root:storage ${pkgdir}/usr/bin/program
}
"""

    def test_nonrootowner(self):
        pkgfile = "__namcap_test_fileownership-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.fileownership.package)
        self.assertEqual(r.errors, [("incorrect-owner %s (%s:%s)", ("usr/bin/program", "root", "storage"))])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.extravars
from Namcap.tests.pkgbuild_test import PkgbuildTest


class ExtravarsTest(PkgbuildTest):
    pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license='GPL-3.0-or-later'
depends=('glibc')
depends+=('glib2')
optdepends=('gtk2')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

mycustomvar="something"
_legalvar="something else"

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr \\
	--libdir=something
  make
  hello=world make something
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  make DESTDIR="${pkgdir}" install
}
"""
    # now a more tricky example
    pkgbuild2 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A pac
kage=1 package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
depends+=('glib2'
glibc=2.12
)
depends_i686=('zlib')
depends_x86_64=('lib32-zlib')
optdepends=('gtk2')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
validpgpkeys=('D7EDD665AB6F255E2EB9BA2702FA998FBAD6674A')
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr \\
	--libdir=something
  _variable=value make; mycustomvar="something"
  hello=world make something
  echo 's/a/b/
        b=c
        s/c=d/'
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  make DESTDIR="${pkgdir}" install
}
"""

    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = Namcap.rules.extravars.package

    def test_example1(self):
        "PKGBUILD with custom variables without underscore"
        os.putenv("environment_pollution", "yes")
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [])
        self.assertEqual(
            set(r.warnings),
            set(
                [
                    ("extra-var-begins-without-underscore %s", "mycustomvar"),
                ]
            ),
        )
        self.assertEqual(r.infos, [])

    def test_example2(self):
        "a tricky PKGBUILD with custom variables without underscore"
        r = self.run_on_pkg(self.pkgbuild2)
        self.assertEqual(r.errors, [])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.anyelf
from Namcap.tests.makepkg import MakepkgTest


class AnyElfTest(MakepkgTest):
    pkgbuild_any = """
pkgname=__namcap_test_anyelf
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
options=(emptydirs)
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/usr/share/directory
  touch ${pkgdir}/usr/share/directory_file
}
"""
    pkgbuild_elf = """
pkgname=__namcap_test_anyelf
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('any')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
options=(emptydirs)
source=()
build() {
  true
}
package() {
  install -m755 -D /bin/ls ${pkgdir}/usr/bin/ls
}
"""

    pkgbuild_static = """
pkgname=__namcap_test_anyelf
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('any')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
options=(emptydirs)
source=()
build() {
  echo "void function() { return; }" > main.c
  gcc -c main.c
  ar cru library.a main.o
}
package() {
  install -m644 -D ${srcdir}/library.a ${pkgdir}/usr/lib/library.a
}
"""

    def test_not_any_no_elf(self):
        pkgfile = "__namcap_test_anyelf-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild_any)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.anyelf.package)
        self.assertEqual(r.errors, [])
        self.assertEqual(r.warnings, [("no-elffiles-not-any-package", ())])
        self.assertEqual(r.infos, [])

    def test_any_but_elf(self):
        pkgfile = "__namcap_test_anyelf-1.0-1-any.pkg.tar"
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild_elf)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.anyelf.package)
        self.assertEqual(r.errors, [("elffile-in-any-package %s", "usr/bin/ls")])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])

    def test_static(self):
        pkgfile = "__namcap_test_anyelf-1.0-1-any.pkg.tar"
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild_static)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.anyelf.package)
        self.assertEqual(r.errors, [("elffile-in-any-package %s", "usr/lib/library.a")])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2024 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os
from Namcap.ruleclass import TarballRule


class dbus1locationRule(TarballRule):
    name = "dbus1location"
    description = "Checks for dbus files in /etc/dbus1/system.d/"

    def analyze(self, _, tar):
        for entry in tar:
            # ignore the actual directory, as that's handled by emptydirs
            if entry.isdir():
                continue

            name = os.path.normpath(entry.name)

            # check for files in /etc/dbus-1/system.d/
            if name.startswith("etc/dbus-1/system.d/"):
                self.warnings.append(("dbus-1-location %s", name))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import re

from Namcap.ruleclass import PkgbuildRule


class package(PkgbuildRule):
    name = "makepkgfunctions"
    description = "Looks for calls to makepkg functionality"

    def analyze(self, pkginfo, pkgbuild):
        bad_calls = ["msg", "msg2", "warning", "error", "plain"]
        regex = re.compile(r"^\s+(%s) " % "|".join(bad_calls))
        hits = set()
        for i in pkginfo.pkgbuild:
            if match := regex.match(i):
                call = match.group(1)
                hits.add(call)
        for i in hits:
            self.warnings.append(("makepkg-function-used %s", (i,)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os
import shutil
import tempfile
import unittest
from typing import Callable, NoReturn

import Namcap.package
import Namcap.ruleclass


class PkgbuildTest(unittest.TestCase):
    """
    This class is the template for unit tests concerning PKGBUILD rules.
    The setUp()	method should set self.rule to a class derived from
    PkgbuildRule.

    The usual way to do that is by defining

    def preSetUp(self):
            self.rule = myRuleClass
            # Add standard valid PKGBUILDs to tests
            self.test_valid = PkgbuildTest.valid_tests
    """

    rule: Callable[..., Namcap.ruleclass.AbstractRule] | NoReturn

    def preSetUp(self):
        pass

    def setUp(self):
        self.preSetUp()
        self.tmpdir = tempfile.mkdtemp()
        self.tmpname = os.path.join(self.tmpdir, "PKGBUILD")

    def run_on_pkg(self, p):
        with open(self.tmpname, "w", encoding="utf-8") as f:
            f.write(p)
        pkginfo = Namcap.package.load_from_pkgbuild(self.tmpname)
        r = self.rule()
        if isinstance(r, Namcap.ruleclass.PkgInfoRule):
            if pkginfo.is_split:
                for p in pkginfo.subpackages:
                    r.analyze(p, self.tmpname)
            else:
                r.analyze(pkginfo, self.tmpname)
        if isinstance(r, Namcap.ruleclass.PkgbuildRule):
            r.analyze(pkginfo, self.tmpname)
        return r

    def valid_tests(self):
        "Valid PKGBUILDS"
        for p in valid_pkgbuilds:
            r = self.run_on_pkg(p)
            self.assertEqual(r.errors, [])
            self.assertEqual(r.warnings, [])
            self.assertEqual(r.errors, [])

    def tearDown(self):
        shutil.rmtree(self.tmpdir)


_basic = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
epoch=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc' 'pacman')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  make DESTDIR="${pkgdir}" install
}
"""

_split = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgbase=mysplitpackage
pkgname=('mypackage1' 'mypackage2' 'mypackage3')
pkgver=1.0
pkgrel=1
epoch=2
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
makedepends=('make' 'python')
checkdepends=('perl')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgbase}-${pkgver}
  ./configure --prefix=/usr
  make
}

check() {
  cd "${srcdir}"/${pkgbase}-${pkgver}
  make check
}

package_mypackage1() {
  pkgdesc="Package 1"
  depends=("python")
  cd "${srcdir}"/${pkgbase}-${pkgver}
  make DESTDIR="${pkgdir}" install1
}

package_mypackage2() {
  pkgdesc="Package 2"
  depends=("zlib" "mypackage1")
  cd "${srcdir}"/${pkgbase}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install2
}

package_mypackage3() {
  pkgdesc="Package 3"
  depends=("zlib")
  optdepends=("mypackage1: for foobar functionality")
  install=somescript.install
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install3
}
"""

valid_pkgbuilds = [_basic, _split]



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

from Namcap.ruleclass import TarballRule


class package(TarballRule):
    name = "emptydir"
    description = "Warns about empty directories in a package"

    def analyze(self, pkginfo, tar):
        dirs = []
        entries = []
        for entry in tar:
            if entry.isdir():
                dirs.append(entry.name.rstrip("/"))
            entries.append(entry.name.rstrip("/"))
        nonemptydirs = [os.path.dirname(x) for x in entries]
        self.warnings = [("empty-directory %s", d) for d in (set(dirs) - set(nonemptydirs))]



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import stat

from Namcap.ruleclass import TarballRule


class package(TarballRule):
    name = "permissions"
    description = "Checks file permissions."

    def analyze(self, pkginfo, tar):
        for i in tar.getmembers():
            if not i.mode & stat.S_IROTH and not (i.issym() or i.islnk()):
                self.warnings.append(("file-not-world-readable %s", i.name))
            if i.mode & stat.S_IWOTH and not (i.issym() or i.islnk()):
                self.warnings.append(("file-world-writable %s", i.name))
            if not i.mode & stat.S_IXOTH and i.isdir():
                self.warnings.append(("directory-not-world-executable %s", i.name))
            if str(i.name).endswith(".a") and not (i.issym() or i.islnk()):
                if i.mode != 0o644 and i.mode != 0o444:
                    self.warnings.append(("incorrect-library-permissions %s", i.name))
            if i.mode & (stat.S_ISUID | stat.S_ISGID):
                self.warnings.append(("file-setugid %s", i.name))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"""Checks dependencies resulting from linking of shared libraries."""

import os
import re
import subprocess
from collections import defaultdict
from typing import Literal, TypeAlias

from elftools.elf.dynamic import DynamicSection
from elftools.elf.elffile import ELFFile

import Namcap.package
from Namcap.ruleclass import TarballRule
from Namcap.rules.rpath import get_rpaths
from Namcap.rules.runpath import get_runpaths
from Namcap.util import is_elf

Architecture: TypeAlias = Literal["i686", "x86-64"]

libcache: dict[Architecture, dict[str, str]] = {"i686": {}, "x86-64": {}}

_DependsMap: TypeAlias = dict[str, str]
_LibMap: TypeAlias = dict[str, set[str]]
_ProvidesMap: TypeAlias = dict[str, set[str]]


def scanlibs(fileobj, filename, custom_libs, liblist, libdepends, libprovides):
    """
    Find shared libraries in a file-like binary object

    If it depends on a library or provides one, store that library's path.
    """

    if not is_elf(fileobj):
        return {}

    elffile = ELFFile(fileobj)
    for section in elffile.iter_sections():
        if not isinstance(section, DynamicSection):
            continue
        for tag in section.iter_tags():
            bitsize = elffile.elfclass
            match bitsize:
                case 32:
                    architecture: Architecture = "i686"
                case 64:
                    architecture = "x86-64"
            # DT_SONAME means it provides a library
            if tag.entry.d_tag == "DT_SONAME" and os.path.dirname(filename) in ["usr/lib", "usr/lib32"]:
                soname = re.sub(r"\.so.*", ".so", tag.soname)
                soversion = re.sub(r"^.*\.so\.", "", tag.soname)
                libprovides[soname + "=" + soversion + "-" + str(bitsize)].add(filename)
            # DT_NEEDED means shared library
            if tag.entry.d_tag != "DT_NEEDED":
                continue
            libname = tag.needed
            soname = re.sub(r"\.so.*", ".so", libname)
            soversion = re.sub(r"^.*\.so\.", "", libname)
            if libname in custom_libs:
                libpath = custom_libs[libname][1:]
                continue
            try:
                libpath = os.path.abspath(libcache[architecture][libname])[1:]
            except KeyError:
                # We didn't know about the library, so add it for fail later
                libpath = libname
            libdepends[soname + "=" + soversion + "-" + str(bitsize)] = libpath
            liblist[libpath].add(filename)


def finddepends(libdepends):
    """
    Find packages owning a list of libraries

    Returns:
      dependlist -- a dictionary { package => set(libraries) }
      libdependlist -- a dictionary { soname => package }
      orphans -- the list of libraries without owners
      missing_provides -- the list of sonames without providers
    """
    dependlist = defaultdict(set)
    libdependlist = {}
    missing_provides = {}

    actualpath = {}

    knownlibs = set(libdepends)
    foundlibs = set()

    actualpath = dict((j, os.path.realpath("/" + libdepends[j])[1:]) for j in knownlibs)

    # Sometimes packages don't include all so .so, .so.1, .so.1.13, .so.1.13.19 files
    # They rely on ldconfig to create all the symlinks
    # So we will strip off the matching part of the files and use this regexp to match the rest
    so_end = re.compile(r"(\.\d+)*")
    # Whether we should even look at a particular file
    is_so = re.compile(r"\.so")

    for pkg in Namcap.package.get_installed_packages():
        for j, fsize, fmode in pkg.files:
            if not is_so.search(j):
                continue

            for k in knownlibs:
                # File must be an exact match or have the right .so ending numbers
                # i.e. gpm includes libgpm.so and libgpm.so.1.19.0, but everything links to libgpm.so.1
                # We compare find libgpm.so.1.19.0 startswith libgpm.so.1 and .19.0 matches the regexp
                if j == actualpath[k] or (j.startswith(actualpath[k]) and so_end.match(j[len(actualpath[k]) :])):
                    dependlist[pkg.name].add(libdepends[k])
                    foundlibs.add(k)
                    # Check if the dependency can be satisfied by soname
                    if k in pkg.provides:
                        libdependlist[k] = pkg.name
                    else:
                        missing_provides[k] = pkg.name

    orphans = list(knownlibs - foundlibs)
    return dependlist, libdependlist, orphans, missing_provides


def filllibcache() -> None:
    var = subprocess.Popen(
        "ldconfig -p", env={"LANG": "C"}, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
    ).communicate()
    libline = re.compile(r"\s*(.*) \((.*)\) => (.*)")
    for j in var[0].decode("ascii").splitlines():
        g = libline.match(j)
        if g is not None:
            if g.group(2).startswith("libc6,x86-64"):
                libcache["x86-64"][g.group(1)] = g.group(3)
            else:
                # TODO: This is bogus; what do non x86-architectures print?
                libcache["i686"][g.group(1)] = g.group(3)


class SharedLibsRule(TarballRule):
    name = "sodepends"
    description = "Checks dependencies caused by linked shared libraries"

    def analyze(self, pkginfo, tar):
        liblist: _LibMap = defaultdict(set)
        libdepends: _DependsMap = defaultdict(str)
        libprovides: _ProvidesMap = defaultdict(set)
        dependlist = {}
        libdependlist = {}
        missing_provides = {}
        filllibcache()
        os.environ["LC_ALL"] = "C"
        pkg_so_files = ["/" + n for n in tar.getnames() if ".so" in n]

        for entry in tar:
            if not entry.isfile():
                continue
            f = tar.extractfile(entry)
            # find anything that could be rpath related
            rpath_files = {}
            if is_elf(f):
                rpaths = list(get_rpaths(f)) + list(get_runpaths(f))
                f.seek(0)
                for n in pkg_so_files:
                    for rp in rpaths:
                        rp = os.path.normpath(rp.replace("$ORIGIN", "/" + os.path.dirname(entry.path)))
                        if os.path.dirname(n) == rp:
                            rpath_files[os.path.basename(n)] = n
            scanlibs(f, entry.name, rpath_files, liblist, libdepends, libprovides)
            f.close()

        # Ldd all the files and find all the link and script dependencies
        dependlist, libdependlist, orphans, missing_provides = finddepends(libdepends)

        # Filter out internal dependencies
        libdependlist = dict(filter(lambda elem: elem[1] != pkginfo["name"], libdependlist.items()))
        missing_provides = dict(filter(lambda elem: elem[1] != pkginfo["name"], missing_provides.items()))

        # Handle "no package associated" errors
        self.warnings.extend(
            [
                ("library-no-package-associated %s %s", (libdepends[i], str(list(liblist[libdepends[i]]))))
                for i in orphans
            ]
        )

        # Handle when a required soname does not provided by the associated package yet
        self.infos.extend(
            [
                ("libdepends-missing-provides %s %s (%s)", (i, missing_provides[i], str(list(liblist[libdepends[i]]))))
                for i in missing_provides
            ]
        )

        # Print link-level deps
        for pkg, libraries in dependlist.items():
            if isinstance(libraries, set):
                files = list(libraries)
                needing = set().union(*[liblist[lib] for lib in libraries])
                reasons = pkginfo.detected_deps.setdefault(pkg, [])
                reasons.append(("libraries-needed %s %s", (str(files), str(list(needing)))))
                self.infos.append(("link-level-dependence %s in %s", (pkg, str(files))))

        # Check for soname dependencies
        for i in libdependlist:
            if i in pkginfo["depends"]:
                self.infos.append(
                    (
                        "libdepends-detected-satisfied %s %s (%s)",
                        (i, libdependlist[i], str(list(liblist[libdepends[i]]))),
                    )
                )
                continue
            if i in pkginfo["optdepends"]:
                self.infos.append(
                    (
                        "libdepends-detected-but-optional %s %s (%s)",
                        (i, libdependlist[i], str(list(liblist[libdepends[i]]))),
                    )
                )
                continue
            self.infos.append(
                (
                    "libdepends-detected-not-included %s %s (%s)",
                    (i, libdependlist[i], str(list(liblist[libdepends[i]]))),
                )
            )

        for i in pkginfo["depends"]:
            if ".so" in i and i not in libdependlist:
                self.warnings.append(("libdepends-not-needed %s", (i,)))
            if i.endswith(".so"):
                self.errors.append(("libdepends-without-version %s", (i,)))

        self.infos.append(
            (
                "libdepends-by-namcap-sight depends=(%s)",
                (" ".join(sorted(set(libdependlist) | set(missing_provides.values()))),),
            )
        )

        # Check provided libraries
        for i in libprovides:
            if i in pkginfo["provides"]:
                self.infos.append(("libprovides-satisfied %s %s", (i, str(list(libprovides[i])))))
                continue
            self.infos.append(("libprovides-unsatisfied %s %s", (i, str(list(libprovides[i])))))

        for i in pkginfo["provides"]:
            if ".so" in i and i not in libprovides:
                self.warnings.append(("libprovides-missing %s", (i,)))
            if i.endswith(".so"):
                self.errors.append(("libprovides-without-version %s", (i,)))

        self.infos.append(("libprovides-by-namcap-sight provides=(%s)", (" ".join(libprovides),)))

        # Check for packages in testing
        for i in dependlist.keys():
            p = Namcap.package.load_testing_package(i)
            q = Namcap.package.load_from_db(i)
            if p is not None and q is not None and p["version"] == q["version"]:
                self.warnings.append(("dependency-is-testing-release %s", (i,)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.shebangdepends
from Namcap.tests.makepkg import MakepkgTest


class ShebangDependsTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_shebangdepends
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('any')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=()
source=()
options=(!purge !zipman)
build() {
  cd "${srcdir}"
  echo -e "#! /usr/bin/env python\nprint('a script')" > python_sample
  echo -e "#!/bin\\xffary/da\\x00ta\ncrash?" > binary_sample
}
package() {
  install -Dm755 "$srcdir/python_sample" "$pkgdir/usr/bin/python_sample"
  install -Dm755 "$srcdir/binary_sample" "$pkgdir/usr/share/binary_sample"
}
"""

    def test_shebangdepends(self):
        "Package with missing python dependency"
        pkgfile = "__namcap_test_shebangdepends-1.0-1-any.pkg.tar"
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(
            os.path.join(self.tmpdir, pkgfile), Namcap.rules.shebangdepends.ShebangDependsRule
        )
        e, w, i = Namcap.depends.analyze_depends(pkg)
        self.assertEqual(
            e,
            [
                (
                    "dependency-detected-not-included %s (%s)",
                    ("python", "programs ['python'] needed in scripts ['usr/bin/python_sample']"),
                )
            ],
        )
        self.assertEqual(w, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"These rules checks basic sanity of package metadata"

import os
import re

from Namcap.ruleclass import PkgbuildRule, PkgInfoRule
from Namcap.util import is_debug


class CapsPkgnameRule(PkgInfoRule):
    name = "capsnamespkg"
    description = "Verifies package name in package does not include upper case letters"

    def analyze(self, pkginfo, tar):
        if re.search(r"[A-Z]", pkginfo["name"]) is not None:
            self.errors.append(("package-name-in-uppercase", ()))


class UrlRule(PkgInfoRule):
    name = "urlpkg"
    description = "Verifies url is included in a package file"

    def analyze(self, pkginfo, tar):
        if "url" not in pkginfo:
            self.errors.append(("missing-url", ()))


class LicenseRule(PkgInfoRule):
    name = "license"
    description = "Verifies license is included in a PKGBUILD"

    def analyze(self, pkginfo, tar):
        if is_debug(pkginfo):
            return
        if "license" not in pkginfo or len(pkginfo["license"]) == 0:
            self.errors.append(("missing-license", ()))


class NonUniqueSourcesRule(PkgbuildRule):
    name = "non-unique-source"
    description = "Verifies the downloaded sources have a unique filename"

    def analyze(self, pkginfo, tar):
        for source_file in pkginfo["source"]:
            source_file = source_file.split("#")[0]
            if "::" not in source_file and re.match(
                r"^[vV]?(([0-9]){8}|([0-9]+\.?)+)\.", os.path.basename(source_file)
            ):
                self.warnings.append(("non-unique-source-name %s", os.path.basename(source_file)))



# Copyright (C) 2024 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import unittest
import Namcap.rules


class RulesTests(unittest.TestCase):
    def test_all_rules_dict(self):
        self.assertNotEqual(Namcap.rules.all_rules, {})



# Copyright (C) 2024 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os
from Namcap.ruleclass import TarballRule


class sphinxbuildcachefilesRule(TarballRule):
    name = "sphinxbuildcachefiles"
    description = "Checks for leftover sphinx-build cached environment files"

    def analyze(self, _pkginfo, tar):
        for entry in tar:
            if entry.isdir():
                continue

            name = os.path.normpath(entry.name)
            if name.endswith("environment.pickle"):
                self.errors.append(("sphinx-build-cache-files", ()))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

from Namcap.rules import fhs
from Namcap.tests.makepkg import MakepkgTest


class FHSTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_nonfhs
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/weird/directory
  touch ${pkgdir}/weird/directory/file
  mkdir -p ${pkgdir}/srv
  touch ${pkgdir}/srv/index.html
  mkdir -p ${pkgdir}/run
  touch ${pkgdir}/run/program.pid
}
"""

    def test_nonfhs(self):
        "Package with files in /weird"
        pkgfile = "__namcap_test_nonfhs-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), fhs.FHSRule)
        self.assertEqual(set(r.errors), set([("file-in-temporary-dir %s", "run/program.pid")]))
        self.assertEqual(
            set(r.warnings),
            set(
                [
                    ("file-in-non-standard-dir %s", "srv/"),
                    ("file-in-non-standard-dir %s", "srv/index.html"),
                    ("file-in-non-standard-dir %s", "weird/"),
                    ("file-in-non-standard-dir %s", "weird/directory/"),
                    ("file-in-non-standard-dir %s", "weird/directory/file"),
                ]
            ),
        )
        self.assertEqual(r.infos, [])

    pkgbuild_man = """
pkgname=__namcap_test_nonfhs
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/usr/man
  touch ${pkgdir}/usr/man/something.1
}
"""

    def test_nonfhsman(self):
        "Package with man pages in /usr/man"
        pkgfile = "__namcap_test_nonfhs-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild_man)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), fhs.FHSManpagesRule)
        self.assertEqual(r.errors, [("non-fhs-man-page %s", "usr/man/something.1.gz")])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])

    pkgbuild_info = """
pkgname=__namcap_test_nonfhs
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/usr/info
  touch ${pkgdir}/usr/info/something
}
"""

    def test_nonfhsinfo(self):
        "Package with info pages in /usr/info"
        pkgfile = "__namcap_test_nonfhs-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild_info)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), fhs.FHSInfoPagesRule)
        self.assertEqual(r.errors, [("non-fhs-info-page %s", "usr/info/something.gz")])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

from Namcap.package import load_from_db
from Namcap.ruleclass import TarballRule
from Namcap.util import is_debug


class package(TarballRule):
    name = "symlink"
    description = "Checks that symlinks point to the right place"

    def analyze(self, pkginfo, tar):
        filenames = set(s.name.rstrip("/") for s in tar)
        depfilenames = set()
        for d in pkginfo["depends"]:
            p = load_from_db(d)
            if not p:
                continue
            depfilenames |= set(name.rstrip("/") for name, _, _ in p["files"])
        # debug package needs the corresponding binary packages
        if is_debug(pkginfo):
            for d in [pkginfo["name"]] + pkginfo["provides"]:
                p = load_from_db(d[: -len("-debug")])
                if not p:
                    continue
                depfilenames |= set(name.rstrip("/") for name, _, _ in p["files"])
        filenames |= depfilenames
        for i in tar:
            if i.issym():
                self.infos.append(("symlink-found %s points to %s", (i.name, i.linkname)))
                linktarget = i.linkname
                linklead = os.path.dirname(i.name)
                # os.path.join drops the 1st arg if the 2nd one is absolute
                linkdest = os.path.join(linklead, linktarget)
                linkdest = os.path.normpath(linkdest).lstrip("/").rstrip("/")
                if linkdest not in filenames:
                    self.errors.append(("dangling-symlink %s points to %s", (i.name, i.linkname)))
            if i.islnk():
                self.infos.append(("hardlink-found %s points to %s", (i.name, i.linkname)))
                if i.linkname not in filenames:
                    self.errors.append(("dangling-hardlink %s points to %s", (i.name, i.linkname)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os
from unittest.mock import patch

import Namcap.rules.qmldepends
from Namcap.package import pyalpm_handle
from Namcap.tests.makepkg import MakepkgTest


class _DbWithout:
    """Test double of the local package database which pretends that
    a given package is not installed locally, even if it really is."""

    def __init__(self, pkgname):
        self.pkgcache = [
            alpmPackage for alpmPackage in pyalpm_handle.get_localdb().pkgcache if alpmPackage.name != pkgname
        ]


class _AlpmWithout:
    """Test double of the PyAlpm handle which pretends that a given
    package is not installed locally, even if it really is."""

    def __init__(self, pkgname):
        self.load_pkg = pyalpm_handle.load_pkg
        self.localdb = _DbWithout(pkgname)

    def get_localdb(self):
        return self.localdb


class QmlDependsTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_qmldepends
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('any')
url="http://www.example.com/"
license=('GPL')
depends=()
source=()
build() {
  echo "import QtQuick.Window 2.0" > main.qml
  echo "import QtTest" > not-a-qml-file
}
package() {
  install -Dm 644 main.qml not-a-qml-file -t "$pkgdir"/usr/bin
}
"""

    # This test assumes that the `qt6-declarative` package is not installed.
    # Temporarily stub the local package database to ensure that condition
    # even if the host system really has the package installed.
    @patch.object(Namcap.package, "pyalpm_handle", _AlpmWithout("qt6-declarative"))
    def test_qmldepends(self):
        "Package with missing pacman dependency"
        pkgfile = "__namcap_test_qmldepends-1.0-1-any.pkg.tar"
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.qmldepends.QmlDependencyRule)
        self.assertEqual(
            r.warnings,
            [("qml-module-no-package-associated %s %s", ("QtQuick.Window", "['usr/bin/main.qml']"))],
        )
        self.assertEqual(r.errors, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

from os.path import dirname

from Namcap.ruleclass import TarballRule


class package(TarballRule):
    name = "hardlinks"
    description = "Look for cross-directory/partition hard links"

    def analyze(self, pkginfo, tar):
        hardlinks = [i for i in tar.getmembers() if i.islnk() is True]
        for hardlink in hardlinks:
            if dirname(hardlink.name) != dirname(hardlink.linkname):
                self.errors.append(("cross-dir-hardlink %s %s", (hardlink.name, hardlink.linkname)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"""Verifies that array variables are actually arrays"""

import re

from Namcap.ruleclass import PkgbuildRule


class package(PkgbuildRule):
    name = "array"
    description = "Verifies that array variables are actually arrays"

    def analyze(self, pkginfo, tar):
        arrayvars = [
            "arch",
            "license",
            "groups",
            "depends",
            "makedepends",
            "optdepends",
            "checkdepends",
            "provides",
            "conflicts",
            "replaces",
            "backup",
            "options",
            "source",
            "noextract",
            "md5sums",
            "sha1sums",
            "sha224sums",
            "sha256sums",
            "sha384sums",
            "sha512sums",
            "b2sums",
            "validpgpkeys",
        ]
        for i in pkginfo.pkgbuild:
            m = re.match(r"\s*(.*)\s*=\s*(.*)$", i)
            for j in arrayvars:
                if m and m.group(1) == j:
                    if not m.group(2).startswith("("):
                        self.warnings.append(("variable-not-array %s", (j,)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.hardlinks
from Namcap.tests.makepkg import MakepkgTest


class HardlinksTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_hardlinks
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p "${pkgdir}/usr/bin"
  mkdir -p "${pkgdir}/usr/sbin"
  touch "${pkgdir}/usr/bin/prog1"
  ln "${pkgdir}/usr/bin/prog1" "${pkgdir}/usr/sbin/prog2"
}
"""

    def test_crossdir_hardlink(self):
        "Package with hard links crossing directories"
        pkgfile = "__namcap_test_hardlinks-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.hardlinks.package)
        errors = [(s, set(tup)) for (s, tup) in r.errors]
        self.assertEqual(errors, [("cross-dir-hardlink %s %s", set(("usr/bin/prog1", "usr/sbin/prog2")))])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import re

from Namcap.ruleclass import TarballRule


class package(TarballRule):
    name = "scrollkeeper"
    description = "Verifies that there aren't any scrollkeeper directories."

    def analyze(self, pkginfo, tar):
        scroll = re.compile(r"var.*/scrollkeeper/?$")
        for i in tar.getnames():
            if scroll.search(i):
                self.errors.append(("scrollkeeper-dir-exists %s", i))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

"""
This contains a collection of essentially one-line rules:
If a certain path is detected then a certain dependency is expected.

Anything fancier than this should get its own rule.
"""

import re

from Namcap.ruleclass import TarballRule


class PathDependsRule(TarballRule):
    name = "pathdepends"
    description = "Check for simple implicit path dependencies"
    # list of path regex, dep name, reason tag
    subrules = [
        {"path": r"^usr/share/glib-2\.0/schemas$", "dep": "dconf", "reason": "dconf-needed-for-glib-schemas"},
        {"path": r"^usr/lib/gio/modules/.*\.so$", "dep": "glib2", "reason": "glib2-needed-for-gio-modules"},
        {
            "path": r"^usr/share/icons/hicolor$",
            "dep": "hicolor-icon-theme",
            "reason": "hicolor-icon-theme-needed-for-hicolor-dir",
        },
    ]

    def analyze(self, pkginfo, tar):
        names = [entry.name for entry in tar]
        for subrule in self.subrules:
            pattern = re.compile(subrule["path"])
            if any(pattern.search(n) for n in names):
                dep = subrule["dep"]
                reason = subrule["reason"]
                pkginfo.detected_deps[dep].append((reason, ()))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import re


def _file_has_magic(fileobj, magic_bytes):
    length = len(magic_bytes)
    magic = fileobj.read(length)
    fileobj.seek(0)
    return magic == magic_bytes


def is_elf(fileobj):
    "Take file object, peek at the magic bytes to check if ELF file."
    return _file_has_magic(fileobj, b"\x7fELF")


def is_static(fileobj):
    "Take file object, peek at the magic bytes to check if static lib."
    return _file_has_magic(fileobj, b"!<arch>\n")


def is_script(fileobj):
    "Take file object, peek at the magic bytes to check if script."
    return _file_has_magic(fileobj, b"#!")


def is_java(fileobj):
    "Take file object, peek at the magic bytes to check if class file."
    return _file_has_magic(fileobj, b"\xca\xfe\xba\xbe")


def script_type(fileobj):
    firstline = fileobj.readline()
    fileobj.seek(0)
    try:
        firstline = firstline.decode("utf-8", "strict")
    except UnicodeDecodeError:
        return None
    if not firstline:
        return None
    script = re.compile(r"#!.*/(.+)")
    m = script.match(firstline)
    if m is None:
        return None
    cmd = m.group(1).split()
    name = cmd[0]
    if name == "env":
        name = cmd[1]
    return name


def is_debug(pkginfo):
    "Take pkginfo, checks if it's a debug package"
    return "pkgdesc" in pkginfo and pkginfo["pkgdesc"].startswith("Detached debugging symbols for ")



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

from Namcap.tests.pkgbuild_test import PkgbuildTest
from Namcap.rules.pkginfo import CapsPkgnameRule, LicenseRule, NonUniqueSourcesRule, UrlRule


class NamcapInvalidPkgNameTest(PkgbuildTest):
    pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=MyPackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""

    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = CapsPkgnameRule

    def test_example1(self):
        # Example 1
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [("package-name-in-uppercase", ())])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])


class NamcapLicenseTest(PkgbuildTest):
    pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
depends=('glibc')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""
    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = LicenseRule

    def test_example1(self):
        # Example 1
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [("missing-license", ())])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])


class NamcapUrlTest(PkgbuildTest):
    pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
depends=('glibc')
license=('GPL-3.0-or-later')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""

    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = UrlRule

    def test_example1(self):
        # Example 1
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [("missing-url", ())])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])


class NamcapNonUniqueSourceTest(PkgbuildTest):
    pkgbuild1 = """
pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('x86_64')
depends=('glibc')
license=('GPL-3.0-or-later')
source=("https://github.com/foobar/software/archive/refs/tags/v${pkgver}.tar.gz")
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
}
"""

    pkgbuild2 = """
pkgname=secondpackage
pkgver=1.0
pkgrel=1
pkgdesc="B package"
arch=('x86_64')
depends=('glibc')
license=('GPL-3.0-or-later')
source=("git+https://salsa.debian.org/debian/distro-info-data.git#tag=debian/$pkgver")
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
}
"""

    test_valid = PkgbuildTest.valid_tests

    def preSetUp(self):
        self.rule = NonUniqueSourcesRule

    def test_example1(self):
        r = self.run_on_pkg(self.pkgbuild1)
        self.assertEqual(r.errors, [])
        self.assertEqual(r.warnings, [("non-unique-source-name %s", ("v1.0.tar.gz"))])
        self.assertEqual(r.infos, [])

        r = self.run_on_pkg(self.pkgbuild2)
        self.assertEqual(r.errors, [])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

from Namcap.ruleclass import TarballRule


class ExternalHooksRule(TarballRule):
    name = "externalhooks"
    description = "Check the .INSTALL for commands covered by hooks"
    hooked = [
        "update-desktop-database",
        "update-mime-database",
        "install-info",
        "glib-compile-schemes",
        "gtk-update-icon-cache",
        "xdg-icon-resource",
        "gconfpkg",
        "gio-querymodules",
        "fc-cache",
        "mkfontscale",
        "mkfontdir",
        "systemd-sysusers",
        "systemd-tmpfiles",
        "vlc-cache-gen",
    ]

    def analyze(self, pkginfo, tar):
        if ".INSTALL" not in tar.getnames():
            return
        f = tar.extractfile(".INSTALL")
        text = f.read().decode("utf-8", "ignore")
        f.close()
        for command in self.hooked:
            if command in text:
                self.warnings.append(("external-hooks %s", (command,)))



# Copyright (C) 2003-2023 Namcap contributors, see AUTHORS for details.
# SPDX-License-Identifier: GPL-2.0-or-later

import os

import Namcap.rules.scrollkeeper
from Namcap.tests.makepkg import MakepkgTest


class ScrollkeeperTest(MakepkgTest):
    pkgbuild = """
pkgname=__namcap_test_scrollkeeper
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL-3.0-or-later')
depends=('glibc')
source=()
options=(!purge !zipman)
build() {
  true
}
package() {
  mkdir -p "${pkgdir}/var/lib/scrollkeeper"
  touch "${pkgdir}/var/lib/scrollkeeper/some_file"
}
"""

    def test_scrollkeeper_files(self):
        pkgfile = "__namcap_test_scrollkeeper-1.0-1-%(arch)s.pkg.tar" % {"arch": self.arch}
        with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
            f.write(self.pkgbuild)
        self.run_makepkg()
        pkg, r = self.run_rule_on_tarball(os.path.join(self.tmpdir, pkgfile), Namcap.rules.scrollkeeper.package)
        self.assertEqual(r.errors, [("scrollkeeper-dir-exists %s", "var/lib/scrollkeeper")])
        self.assertEqual(r.warnings, [])
        self.assertEqual(r.infos, [])
